{%- macro first(items) -%}
  {%- for item in items -%}
    {%- if item.startswith('(*)') -%}
      {{ caller(item.replace('(*)', '') | trim) }}
      {%- break -%}
    {%- endif -%}
  {%- endfor -%}
{%- endmacro -%}
